//
//  CakeMobileTracking.swift
//  SwiftCakeSDK
//
//  Created by Brian McCullough on 4/19/16.
//  Copyright © 2016 Accelerize, Inc. All rights reserved.
//

import Foundation
import SystemConfiguration
import AdSupport

var deepLinkView = String()

class CakeMobileTracking: NSObject, NSXMLParserDelegate {
	
	let sdkVersion: String = "swift_1.0.0"
	let userDefaultsFileName: String
	var userId: String = ""
	
	init (userDefaultsFileName: String) {
		self.userDefaultsFileName = userDefaultsFileName
	}
	
	private func errorOutput(errorMessage: NSError) -> Void {
		// By default, prints error to console. Define your own
		// error output here such as loggin or email.
		print(errorMessage)
	}
	
	private func getIdfa() -> String {
		let myIDFA: String
		
		if ASIdentifierManager.sharedManager().advertisingTrackingEnabled {
			myIDFA = ASIdentifierManager.sharedManager().advertisingIdentifier.UUIDString
		} else {
			myIDFA = ""
		}
		
		return myIDFA
	}
	
	private func httpPostBack(finalPostBackURL: String) {
		let url:NSURL = NSURL(string: finalPostBackURL)!
		let session = NSURLSession.sharedSession()
		
		let request = NSMutableURLRequest(URL: url)
		request.HTTPMethod = "POST"
		request.cachePolicy = NSURLRequestCachePolicy.ReloadIgnoringCacheData
		
		let task = session.dataTaskWithRequest(request) {
			(let data, let response, let error) in
			
			guard let _:NSData = data, let _:NSURLResponse = response where error == nil else {
				self.errorOutput(error!)
				return
			}
			
			let xmlResponse = NSString(data: data!, encoding: NSUTF8StringEncoding)
			print(xmlResponse!)
			
			let xmlParser = CakeXmlParser()
			xmlParser.startParsing(data!)
			getDeepLink(xmlParser.dlink as String)
		}
		
		task.resume()
	}
	
	private func postToNSUserDefaults(lineItem: Array<String>) {
		NSUserDefaults.standardUserDefaults().setObject(lineItem, forKey: userDefaultsFileName)
		NSUserDefaults.standardUserDefaults().synchronize()
	}
	
	private func buildPostBackString(baseUrl: String) -> String {
		let rootURL: String = baseUrl
		let udid: String = "&udid=" + getIdfa()
		let userid: String = "&userid=" + userId
		let sdkv: String = "&sdkv=" + sdkVersion
		let finalPostBackURL: String = rootURL + udid + userid + sdkv
		
		return finalPostBackURL
	}
	
	private func internetConnection() -> Bool {
		let reachability: Reachability = Reachability.reachabilityForInternetConnection()
		
		if reachability.currentReachabilityStatus() != NotReachable {
			return true
		} else {
			return false
		}
	}
	
	func sendEvent(baseUrl: String, allowDuplicates: Bool) {
		
		let finalPostBackURL = buildPostBackString(baseUrl)
		let connectedToInternet: Bool = internetConnection()
		let baseUrlArrayItem: Array<String> = [baseUrl]
		let previouslyPostedEvents = NSUserDefaults.standardUserDefaults().objectForKey(userDefaultsFileName)
		
		// If duplicates are allowed, send the event straight through:
		if connectedToInternet == true && allowDuplicates == true {
			httpPostBack(finalPostBackURL)
		}
		
		// Non-recurring event handler:
		if connectedToInternet == true && allowDuplicates == false {
			
			// If empty, okay to send event:
			if previouslyPostedEvents == nil {
				httpPostBack(finalPostBackURL)
				postToNSUserDefaults(baseUrlArrayItem)
			} else {
				var eventURLs = previouslyPostedEvents as! [String]
				let contained = eventURLs.contains(baseUrl)
				
				if contained == false {
					
					httpPostBack(finalPostBackURL)
					eventURLs.append(baseUrl)
					postToNSUserDefaults(eventURLs)
				}
			}
		}
	}
}